<?php


function _wpr_newsletter_form_validate(&$info,&$errors,$whetherToValidateNameUniqueness=true)
{
    $errors = array();
    if (empty($_POST['name']))
    {
        $errors[] = __("The newsletter name field was left empty. Please fill it in to continue.",'wpr_autoresponder');
        $info['name'] = '';
    }
    else if ($whetherToValidateNameUniqueness===true && checkIfNewsletterNameExists($_POST['name']))
    {
        $errors[] = __("A newsletter with that name already exists. ",'wpr_autoresponder');
        $info['name'] = '';
    }
    else
    {
        $info['name'] = $_POST['name'];
    }
    if (empty($_POST['fromname']))
    {
        $errors[] = __("The 'From Name' field was left empty. Please fill it in to continue. ",'wpr_autoresponder');
        $info['fromname'] = '';
    }
    else
    {
        $info['fromname'] = $_POST['fromname'];
    }

     if (empty($_POST['fromemail']))
     {
        $errors[] = __("The 'From Email' field was left empty. Please fill it in to continue. ",'wpr_autoresponder');
        $info['fromemail'] = '';
     }
     else if (!validateEmail($_POST['fromemail']))
     {
         $errors[] = __("The email address provided for 'From Email' is not a valid e-mail address. Please enter a valid email address.",'wpr_autoresponder');
         $info['fromemail'] = '';
     }
     else
     {
         $info['fromemail'] = $_POST['fromemail'];
     }

     if (empty($_POST['reply_to']))
     {
         $errors[] = _("The 'Reply-To' field was left empty. Please fill in an email address in the reply-to field.");
         $info['reply_to'] ='';
     }
     else if (!validateEmail($_POST['reply_to']))
     {
         $errors[] = _("The 'Reply-To' field was filled with an invalid e-mail address. Please fill in a valid email address.");
         $info['reply_to'] ='';
     }
     else
     {
        $info['reply_to'] = $_POST['reply_to'];
     }
    
     $info['id'] = $_POST['id'];
     
     $info['description'] = $_POST['description'];
     $info['fromname'] = $_POST['fromname'];
     $info['fromemail'] = $_POST['fromemail'];

     $info = apply_filters("_wpr_newsletter_form_validation",$info);
     $errors = apply_filters("_wpr_newsletter_form_validation_errors",$errors);

}

function _wpr_newsletter_edit()
{
    _wpr_setview("newsletter_form");

    if (!_wpr_isset("parameters"))
    {
        $id = $_GET['nid'];
        $newsletter = _wpr_newsletter_get($id);
        _wpr_set("parameters",$newsletter);
    }    
    _wpr_set("heading",__("Edit Newsletter",'wpr_autoresponder'));
    _wpr_set("edit",true);
    _wpr_set("button_text",__("Save Changes",'wpr_autoresponder'));
    _wpr_set("wpr_form","newsletter_edit_form");
}

function _wpr_newsletter_edit_form_post_handler()
{
    $info = array();
    $errors = array();
   
    _wpr_newsletter_form_validate($info,$errors,false);

    if (count($errors) ===0)
    {
        _wpr_newsletter_update($info);
        $newsletter_home = "admin.php?page=_wpr/newsletter";
        wp_redirect($newsletter_home);
        exit;
    }
    $info = (object) $info;
    _wpr_set("parameters",$info);
    _wpr_set("errors",$errors);
}


function _wpr_newsletter_create_form_post_handler()
{
    $info = array();
    $errors = array();

    _wpr_newsletter_form_validate($info,$errors);

    if (count($errors) ===0)
    {
        _wpr_newsletter_create($info);
        $newsletter_home = "admin.php?page=_wpr/newsletter";
        wp_redirect($newsletter_home);
        exit;
    }
    $info = (object) $info;
    _wpr_set("parameters",$info);
    _wpr_set("errors",$errors);

}


function checkIfNewsletterNameExists($name)
{
    $name = trim($name);
    global $wpdb;
    $query = "SELECT COUNT(*) num FROM ".$wpdb->prefix."wpr_newsletters where name='$name'";
    $results = $wpdb->get_results($query);
    return ($results[0]->num !=0); 
}


function _wpr_newsletter_add()
{
    _wpr_setview("newsletter_form");
    if (!_wpr_isset("parameters"))
    {
        $id = $_GET['nid'];
        $newsletter = _wpr_newsletter_get($id);
        _wpr_set("parameters",$newsletter);
    }
    _wpr_set("heading",__("Create Newsletter",'wpr_autoresponder'));
    _wpr_set("edit",false);
    _wpr_set("button_text",__("Create Newsletter",'wpr_autoresponder'));
    _wpr_set("wpr_form","newsletter_create_form");
}

function _wpr_newsletter_home()
{
    global $wpdb;
     _wpr_set("_wpr_view","newsletter_home");
    $getNewslettersListQuery = "SELECT id, name, reply_to, fromname, fromemail FROM ".$wpdb->prefix."wpr_newsletters";
    $newsletterList = $wpdb->get_results($getNewslettersListQuery);
    $newsletterList = apply_filters("_wpr_newsletter_home_list",$newsletterList);
    _wpr_set("newsletterList",$newsletterList);

}


function _wpr_newsletter_handler()
{
    $action = (isset($_GET['act']))?$_GET['act']:null;

	switch ($action)
	{
            case 'add':
                _wpr_newsletter_add();
                break;
            case 'edit':
                _wpr_newsletter_edit();
                    break;
            case 'delete':
               _wpr_newsletter_delete();
			   break;
            case 'forms':
               _wpr_newsletter_forms();
			   break;
             default:
                 _wpr_newsletter_home();
	}
}



function _wpr_newsletter_get($id)
{
	global $wpdb;

	$query = "SELECT * FROM ".$wpdb->prefix."wpr_newsletters where id=$id";

	$result = $wpdb->get_results($query);

	return $result[0];



}

function _wpr_newsletters_get()
{
	global $wpdb;
	$query = "SELECT * FROM ".$wpdb->prefix."wpr_newsletters";
	$newslettersResult = $wpdb->get_results($query);
	return $newslettersResult;
}

function _wpr_newsletter_update($info)

{

	global $wpdb;

	$info = (object) $info;

	$query = "UPDATE  ".$wpdb->prefix."wpr_newsletters SET name='$info->name', reply_to='$info->reply_to', description='$info->description', `fromname`='$info->fromname', `fromemail`='$info->fromemail' where id='$info->id';";

	$result = $wpdb->query($query);
}

function _wpr_newsletter_create($info)
{
	global $wpdb;

	$info = (object) $info;


	$query = "INSERT INTO ".$wpdb->prefix."wpr_newsletters (name,reply_to, description, fromname, fromemail) values ('$info->name','$info->reply_to','$info->description','$info->fromname','$info->fromemail');";

	$wpdb->query($query);
}

function _wpr_get_newsletters()
{
	global $wpdb;
	$query = "SELECT * FROM ".$wpdb->prefix."wpr_newsletters";
	$newsletters = $wpdb->get_results($query);
	if (count($newsletters) > 0)
		return $newsletters;
	else
		return false;
}

function _wpr_newsletter_delete()
{
	global $wpdb;
	$prefix = $wpdb->prefix;
	$nid = $_GET['nid'];
	try {
		$newsletter = Newsletter::getNewsletter($nid);
	}
	catch (Exception $excp)
	{
		 _wpr_set("_wpr_view","newsletter_delete_not_found");
		 return;		
	}	
	
	if (isset($_GET['confirmed']) && $_GET['confirmed'] == 'true')
	{
		$newsletter->delete();
		$newsletter_home = "admin.php?page=_wpr/newsletter";
		wp_redirect($newsletter_home);
	}
		
	$getEmailsPendingDeliveryQuery = sprintf("SELECT COUNT(*) number FROM %swpr_queue WHERE sid=(SELECT id FROM %swpr_subscribers WHERE nid=%d) AND sent=0;",$prefix,$prefix,$nid);
	$emails_pending_count_result = $wpdb->get_results($getEmailsPendingDeliveryQuery);
	$number_pending = $emails_pending_count_result[0]->number;
	
	_wpr_set("newsletter_name",$newsletter->getName());
	_wpr_set("subscriber_count",$newsletter->getNumberOfActiveSubscribers());
	_wpr_set("_wpr_view","newsletter_delete");
	_wpr_set("nid",$nid);
	_wpr_set("emailsPendingDelivery",$number_pending);
}
